﻿//-----------------------------------------------------------------------------
//    This file is part of ds30 Loader.
//
//    ds30 Loader is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation.
//
//    ds30 Loader is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with ds30 Loader.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------  

using System;
using System.Text;
using System.IO;
using GHelper;


namespace ds30Loader
{
	//-------------------------------------------------------------------------
	// Class: clsHex
	//-------------------------------------------------------------------------
	public class clsHex
	{
        public enum HexRecordType{ Data=0, EndOfFile, ExtendedSegmentAddress, StartSegmentAddress, ExtendedLinearAddress, StartLinearAddress };
         
		//---------------------------------------------------------------------
		// Variables
		//---------------------------------------------------------------------
        protected string strFilename = "";
		
        protected bool bIsParsed = false;        
        
        public clsParseSettings objParsedSettings;

        protected bool bHasValidProgram = false;
		protected bool bHasValidEeprom = false;
		protected bool bHasValidConfigs = false;
        

		// These are filled in by ParseHexFile(), not all families use all of theese	
        public int [] iProgMem;		

        protected int iProgPagesUsed;
        public bool [] bProgPageUsed;

		public bool [] bProgRowUsed;		        
        protected int iProgRowsUsed;	
	
		protected int iProgWordsUsed;

		public int [] iEEMem;
		public bool [] bEEWordUsed;
		protected int iEEWordsUsed;
				
		public int [] iConfigMem;
		public bool [] bConfigWordUsed;		
		protected int iConfigWordsUsed;

		// 
		public delegate void HexFileValidateDelegate( object sender, clsHexFileValidateEventArgs e );
		public event HexFileValidateDelegate HexFileValidate;	

		// 
		public delegate void HexFileParseDelegate( object sender, clsHexFileParseEventArgs e );
		public event HexFileParseDelegate HexFileParse;	


        //---------------------------------------------------------------------
		// Constructor()
		//---------------------------------------------------------------------
		public clsHex()
		{
            objParsedSettings = new clsParseSettings();
		}// Constructor()


		//---------------------------------------------------------------------
		// Constructor()
		//---------------------------------------------------------------------
		public clsHex( string pstrFilename )
		{
            objParsedSettings = new clsParseSettings();
            strFilename = pstrFilename;
		}// Constructor()
		
						        
		//---------------------------------------------------------------------
		// Event: HexFileParse
		//---------------------------------------------------------------------
		protected internal void OnHexFileParse( clsHexFileParseEventArgs e )
		{
			// Event will be null if no client has hooked up a delegate to the event
			if ( HexFileParse != null ) {
				HexFileParse( this, e );
			}
		}//Event: HexFileParse	
		
		        
		//---------------------------------------------------------------------
		// Event: FileValidate
		//---------------------------------------------------------------------
		protected internal void OnHexFileValidate( clsHexFileValidateEventArgs e )
		{
			// Event will be null if no client has hooked up a delegate to the event
			if ( HexFileValidate != null ) {
				HexFileValidate( this, e );
			}
		}//Event: FileValidated

 		  		
        //---------------------------------------------------------------------
		// Property: configWordsUsed
		//---------------------------------------------------------------------
		public int configWordsUsed
		{
			get {
				return iConfigWordsUsed;
			}
        }//Property: configWordsUsed


 		//---------------------------------------------------------------------
		// Property: eeWordsUsed
		//---------------------------------------------------------------------
		public int eeWordsUsed
		{
			get {
				return iEEWordsUsed;
			}
        }//Property: eeWordsUsed


 		//---------------------------------------------------------------------
		// Property: filename
		//---------------------------------------------------------------------
		public string filename
		{
			get {
				return strFilename;
			} set {
				if ( value != strFilename ) {
					strFilename = value.Trim();
                    
                    bIsParsed = false;
                    bHasValidProgram = false;
		            bHasValidEeprom = false;
		            bHasValidConfigs = false;
				}
			}
        }//Property: filename
       

 		//---------------------------------------------------------------------
		// Property: hasValidProgram
		//---------------------------------------------------------------------
		public bool hasValidProgram
		{
			get {
				return bHasValidProgram;
			}
        }//Property: hasValidProgram

        
 		//---------------------------------------------------------------------
		// Property: hasValidEeprom
		//---------------------------------------------------------------------
		public bool hasValidEeprom
		{
			get {
				return bHasValidEeprom;
			}
        }//Property: hasValidEeprom

        
 		//---------------------------------------------------------------------
		// Property: hasValidConfigs
		//---------------------------------------------------------------------
		public bool hasValidConfigs
		{
			get {
				return bHasValidConfigs;
			}
        }//Property: hasValidConfigs


 		//---------------------------------------------------------------------
		// Property: isParsed
		//---------------------------------------------------------------------
		protected bool isParsed
		{
			get {
				return bIsParsed;
			}
        }//Property: isParsed


 		//---------------------------------------------------------------------
		// Property: progRowsUsed
		//---------------------------------------------------------------------
		public int progRowsUsed
		{
			get {
				return iProgRowsUsed;
			}
        }//Property: progRowsUsed


  		//---------------------------------------------------------------------
		// Property: progWordsUsed
		//---------------------------------------------------------------------
		public int progWordsUsed
		{
			get {
				return iProgWordsUsed;
			}
        }//Property: progWordsUsed


        //---------------------------------------------------------------------
		// BytesToWrite()
		//---------------------------------------------------------------------
		virtual public int BytesToWrite( clsDevice pobjDevice, bool pbWriteProgram, bool pbWriteEeprom, bool pbWriteConfigs )
		{
			return 0;
		}// BytesToWrite()


        //---------------------------------------------------------------------
		// CalcHexContent()
		//---------------------------------------------------------------------
        public void CalcHexContent( clsParseSettings pobjSettings, int piTabLevel )
		{
            //--------------------------------------------------------------------------
            // Find out how many pages are used
            //--------------------------------------------------------------------------   
            {
                iProgPagesUsed = 0;
                for ( int iPageIter = 0; iPageIter < bProgPageUsed.Length; iPageIter++ ) {
                    if ( bProgPageUsed[ iPageIter ] == true ) {
                        ++iProgPagesUsed;
                        if ( clsds30Loader.debugMode == true ) {
                            if ( pobjSettings.device.pageSizeR > 1 ) {
                                OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Flash page " + iPageIter.ToString() + " is used", piTabLevel) );
                            }
                        
                            for ( int iRow = iPageIter * pobjSettings.device.pageSizeR; iRow < iPageIter * pobjSettings.device.pageSizeR + pobjSettings.device.pageSizeR; iRow++ ) {
                                if ( bProgRowUsed[ iRow ] == true ) {
                                    OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Flash row " + iRow.ToString() + " is used", piTabLevel) );
                                }
                            }
                        }
                    }
                }
                if ( iProgPagesUsed == 0 ) {
                    bHasValidProgram = false;
                }
            }


            //--------------------------------------------------------------------------
            // Find out how many program rows are used
            //--------------------------------------------------------------------------
            {
                iProgRowsUsed = 0;
                for ( int iIter = 0; iIter < bProgRowUsed.Length; iIter++ ) {
                    if ( bProgRowUsed[ iIter ] == true ) {
                        ++iProgRowsUsed;
                    }
                }
                if ( iProgRowsUsed == 0 ) {
                    bHasValidProgram = false;
                }
            }
			
			
			//--------------------------------------------------------------------------
			// Eeprom
			//--------------------------------------------------------------------------
		    {
                if ( iEEWordsUsed == 0 ) {
			        bHasValidEeprom = false;
		        }	

                if ( clsds30Loader.debugMode == true && pobjSettings.device.EepromSizeB > 0 ) {
		            for ( int iIter = 0; iIter < bEEWordUsed.Length; iIter++ ) {
			            if ( bEEWordUsed[iIter] == true) { 
                            if ( pobjSettings.device.configWordSizeB == 1 ) {
                                OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Eeprom word " + iIter.ToString() + " is used: 0x" + iEEMem[iIter].ToString("X2"), piTabLevel) );
                            } else if ( pobjSettings.device.configWordSizeB == 2 ) {
				                OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Eeprom word " + iIter.ToString() + " is used: 0x" + ((iEEMem[iIter*2+1]<<8)+iEEMem[iIter*2]).ToString("X4"), piTabLevel) );
                            }
			            }
                    }
                }
            }
	    

			//--------------------------------------------------------------------------
			// Configs
			//--------------------------------------------------------------------------
 		    {
                if ( iConfigWordsUsed > pobjSettings.device.configCount ) {
			        OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.warning, "hex-file contains more config locations than the device has", piTabLevel) );
			        bHasValidConfigs = false;
		        } else if ( iConfigWordsUsed == 0 ) {
			        bHasValidConfigs = false;
		        }
                
                if ( clsds30Loader.debugMode == true && pobjSettings.device.configCount > 0 ) {
		            for ( int iIter = 0; iIter < bConfigWordUsed.Length; iIter++ ) {
			            if ( bConfigWordUsed[iIter] == true ) { 
				            if ( pobjSettings.device.configWordSizeB == 1 ) {
                                OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Config word " + iIter.ToString() + " is used: 0x" + iConfigMem[iIter].ToString("X2"), piTabLevel) );
                            } else if ( pobjSettings.device.configWordSizeB == 2 ) {
                                OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Config word " + iIter.ToString() + " is used: 0x" + ((iConfigMem[iIter*2+1]<<8)+iConfigMem[iIter*2]).ToString("X4"), piTabLevel) );
                            }
			            }
			        }
                }
            }

        }// CalcHexContent()


        //---------------------------------------------------------------------
		// CheckForBlOverwrite()
		//---------------------------------------------------------------------
		public void CheckForBlOverwrite( clsParseSettings pobjSettings, int piTabLevel )
		{
            bool bBlOverwrite = false;
            
            int iBlStartRow = GetBootloaderStartRow( pobjSettings );
            int iBlEndRow = GetBootloaderEndRow( pobjSettings );

            for ( int iIter = iBlStartRow; iIter <= iBlEndRow; iIter++ ) {
                if ( bProgRowUsed[ iIter ] == true ) {
                    bBlOverwrite = true;
                    break;
                }
            }
			if ( bBlOverwrite == true ) {
                OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.warning, "The hex-file contains code that will overwrite the bootloader", piTabLevel) );
				if ( pobjSettings.allowBlOverwrite == false ) {
					bHasValidProgram = false;
				}
			}
		}// BytesToWrite()  


        //---------------------------------------------------------------------
		// CheckForGotoAt0()
		//---------------------------------------------------------------------
		public void CheckForGotoAt0( clsParseSettings pobjSettings, int piTabLevel )
		{
            string strFamily = pobjSettings.device.family.name;


            //---------------------------------------------------------------------
		    // PIC16F
		    //---------------------------------------------------------------------            
            if ( strFamily == "PIC16F" ) {
                 if ( pobjSettings.noGoto == false ) {
                    bool bGotoFound = 
                        (
                            (iProgMem[1] & 0x38) == 0x28 ||
                            (iProgMem[3] & 0x38) == 0x28 ||
                            (iProgMem[5] & 0x38) == 0x28
                        )
                        &&
                        bProgRowUsed[0] == true
                    ;

                    if ( bGotoFound == false ) {
                        OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.warning, "There's no GOTO at location 0x00", piTabLevel) );
                        bHasValidProgram = false;
                    }
                }               
            

            //---------------------------------------------------------------------
		    // PIC18F  PIC18FJ
		    //---------------------------------------------------------------------            
            } else if ( strFamily == "PIC18F" || strFamily == "PIC18FJ" ) {
                if ( pobjSettings.noGoto == false ) {
                    bool bGotoFound = 
                        iProgMem[1] == 0xEF &&
                        (iProgMem[3] & 0xF0) == 0xF0 &&
                        bProgRowUsed[0] == true
                    ;

                    if ( bGotoFound == false ) {
                        OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.warning, "There's no GOTO at location 0x00", piTabLevel) );
                        bHasValidProgram = false;
                    }
                }
          
                
            //---------------------------------------------------------------------
		    // PIC24F  PIC24FJ  PIC24H  dsPIC30F  dsPIC33FJ
		    //---------------------------------------------------------------------            
            } else { 
                if ( pobjSettings.noGoto == false ) {
                    bool bGotoFound = 
                        iProgMem[0] == 0x04 &&
                        bProgRowUsed[0] == true
                    ;

                    if ( bGotoFound == false ) {
                        OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.warning, "There's no GOTO at location 0x00", piTabLevel) );
                        bHasValidProgram = false;
                    }
                }
            }
        }// CheckForGotoAt0()

        

        //---------------------------------------------------------------------
		// GetBootloaderEndRow()
		//---------------------------------------------------------------------
		public int GetBootloaderEndRow( clsParseSettings pobjParseSettings ) 
		{
			return GetBootloaderStartRow(pobjParseSettings) + pobjParseSettings.blSizeP * pobjParseSettings.device.pageSizeR - 1;
        }// GetBootloaderEndRow() 
       

        //---------------------------------------------------------------------
		// GetBootloaderStartAddress()
		//---------------------------------------------------------------------
		public int GetBootloaderStartAddress( clsParseSettings pobjParseSettings ) 
		{
			clsDevice objDev = pobjParseSettings.device;
            if ( objDev.family.name == "PIC16F" ) {
                return objDev.flashSizeP - pobjParseSettings.blPlacementP * objDev.pageSizeR * objDev.rowsizeW;
            } else {
                return objDev.flashSizeP - pobjParseSettings.blPlacementP * objDev.pageSizeR * objDev.rowsizeW * 2/*word->pcu*/;
            }
        }// GetBootloaderStartAddress()
 		
        
        //---------------------------------------------------------------------
		// GetBootloaderStartRow()
		//---------------------------------------------------------------------
		public int GetBootloaderStartRow( clsParseSettings pobjParseSettings ) 
		{
            if ( pobjParseSettings.device.family.name == "PIC16F" ) {
                return GetBootloaderStartAddress(pobjParseSettings) / ( 1 * pobjParseSettings.device.rowsizeW );
			} else {
                return GetBootloaderStartAddress(pobjParseSettings) / ( 2 * pobjParseSettings.device.rowsizeW );
            }
        }// GetBootloaderStartRow()


        //---------------------------------------------------------------------
		// InitBuffers()
		//---------------------------------------------------------------------
		public void InitBuffers( clsParseSettings pobjSettings, int piTabLevel ) 
		{
            int iIter;
            for ( iIter = 0; iIter < bProgPageUsed.Length; iIter++ ) bProgPageUsed[iIter] = false;
            for ( iIter = 0; iIter < bProgRowUsed.Length; iIter++ ) bProgRowUsed[iIter] = false;
            
            if ( pobjSettings.device.family.name == "PIC16F" ) {
                for ( iIter = 0; iIter < iProgMem.Length; iIter++ ) iProgMem[iIter] = 0x00; //NOP
            } else {
                for ( iIter = 0; iIter < iProgMem.Length; iIter++ ) iProgMem[iIter] = 0xFF; //NOP
            }

            for ( iIter = 0; iIter < bEEWordUsed.Length; iIter++ ) bEEWordUsed[iIter] = false;
		    for ( iIter = 0; iIter < iEEMem.Length; iIter++ ) iEEMem[iIter] = 0xFF;
			
            for ( iIter = 0; iIter < bConfigWordUsed.Length; iIter++ ) bConfigWordUsed[iIter] = false;
		    for ( iIter = 0; iIter < iConfigMem.Length; iIter++ ) iConfigMem[iIter] = 0xFF;	
        }// InitBuffers()


  		//---------------------------------------------------------------------
		// NeedsParsing()
		//---------------------------------------------------------------------
		protected bool NeedsParsing( clsParseSettings pobjSettings )
		{
			return ( pobjSettings.CompareTo(objParsedSettings) == false || isParsed == false );
        }//Property: NeedsParsing()


        //---------------------------------------------------------------------
		// OutputBlDebugInfo()
		//---------------------------------------------------------------------
		public void OutputBlDebugInfo( clsParseSettings pobjSettings, int iTabLevel )
		{
            int iStartRow = GetBootloaderStartRow(pobjSettings);
            int iEndRow = GetBootloaderEndRow(pobjSettings);
            int iStartPage = iStartRow / pobjSettings.device.pageSizeR;
            int iEndPage = iEndRow / pobjSettings.device.pageSizeR;

            OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Bootloader start address: 0x" + GetBootloaderStartAddress(pobjSettings).ToString("X2"), iTabLevel) );
            OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Bootloader start page: " + iStartPage.ToString(), iTabLevel) );
            OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Bootloader end page: " + iEndPage.ToString(), iTabLevel) );
            OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Bootloader start row: " + iStartRow.ToString(), iTabLevel) );
            OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Bootloader end row: " + iEndRow.ToString(), iTabLevel) );
            
        }// OutputBlDebugInfo()
        
        
        //---------------------------------------------------------------------
		// ParseHexFilePIC16()
		//---------------------------------------------------------------------
		public void ParseHexFilePIC16( clsParseSettings pobjSettings, int iTabLevel )
		{
            iProgWordsUsed = 0;
            iEEWordsUsed = 0;
            iConfigWordsUsed = 0;

            //--------------------------------------------------------------------------
			// Hex-file format
			// : 10 8000 00 0800 fa00 000f 8001
			// | |  |    |  |-Data
			// | |  |    |----Line record type
			// | |  |---------Line address
			// | |------------Line byte count
			// |--------------Start code (just a simple colon)
			//--------------------------------------------------------------------------
			int iLineAddrHigh = 0;
            int iAddress;
            int iOutAddr;
			
            string strFileLine;
			string strLineStartCode;
            int iLineByteCount;
            int iLineAddress;
            HexRecordType iLineRecordType;
            string strLineData;

            // Open hex-file
            TextReader textReader = new StreamReader( strFilename );            
 
            while ( textReader.Peek() != -1 ) {
                strFileLine = textReader.ReadLine();
                strLineStartCode = strFileLine.Substring(0, 1);

               if ( strLineStartCode == ":" ) {
                    //Parse the line
                    iLineByteCount = int.Parse( strFileLine.Substring( 1, 2 ), System.Globalization.NumberStyles.HexNumber );
                    iLineAddress = ( int.Parse( strFileLine.Substring( 3, 4 ), System.Globalization.NumberStyles.HexNumber ) & 65535 );
                    iLineRecordType = (HexRecordType)int.Parse( strFileLine.Substring( 7, 2 ), System.Globalization.NumberStyles.HexNumber );
                    strLineData = strFileLine.Substring( 9, ( iLineByteCount * 2 ) );

                    // Calculate address
                    iAddress = (iLineAddrHigh << 16) + iLineAddress;

                    // Data record
                    if ( iLineRecordType == HexRecordType.Data ) {

                        // Regular program memory
                        if ( iAddress <= pobjSettings.device.flashSizeP ) {
							iOutAddr = iAddress;
                            iProgWordsUsed += iLineByteCount/2;

						    for ( int iByte = 0; iByte < iLineByteCount; iByte++ ) {
							    iProgMem[iOutAddr + iByte] = int.Parse( strLineData.Substring(iByte*2 + 0, 2), System.Globalization.NumberStyles.HexNumber );
                                bProgPageUsed[ (iAddress + iByte) / (pobjSettings.device.pageSizeR * pobjSettings.device.rowsizeW * 2)  ] = true;
							    bProgRowUsed[  (iAddress + iByte) / (pobjSettings.device.rowsizeW * 2)  ] = true;
						    }	
                            

						// Eeprom memory
						} else if ( pobjSettings.device.EepromSizeB > 0 && iAddress >= 0x4200 && iAddress < (0x4200 + pobjSettings.device.EepromSizeB) ) {
                            iOutAddr = iAddress - 0x4200;

							for ( int iByte = 0; iByte < iLineByteCount/2; iByte++ ) {
								iEEMem[ iOutAddr + iByte ] = int.Parse( strLineData.Substring(iByte*4 + 0, 2), System.Globalization.NumberStyles.HexNumber );								
								bEEWordUsed[ iOutAddr + iByte ] = true;
                                ++iEEWordsUsed;
							}

							                        
						// Config memory
						} else if ( pobjSettings.device.configCount > 0 && iAddress >= 0x300000 && iAddress <= 0x30000D) {	
							iOutAddr = iAddress - 0x300000;
							
							for ( int iByte = 0; iByte < iLineByteCount; iByte++ ) {
								iConfigMem[ iOutAddr + iByte ] = int.Parse( strLineData.Substring(iByte*2 + 0, 2), System.Globalization.NumberStyles.HexNumber );
								bConfigWordUsed[ iOutAddr + iByte ] = true;
                                ++iConfigWordsUsed;
							}	
                        
                        // Unknown data
                        } else {
                            OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Unknown data found, " + iLineByteCount.ToString() + " bytes at 0x" + iAddress.ToString("X4"), iTabLevel) );
                        }

                    // End Of File record
                    } else if ( iLineRecordType == HexRecordType.EndOfFile ) {
                        break;

                    // Extended Linear Address Record
                    // The two data bytes represent the upper 16 bits of the 32 bit address, when combined with the address of the 00 type record.
                    } else if ( iLineRecordType == HexRecordType.ExtendedLinearAddress ) {
                        iLineAddrHigh = ( int.Parse( strFileLine.Substring( 9, 4 ), System.Globalization.NumberStyles.HexNumber ) & 65535 );
                    }
                }
            }

            // Close files
            textReader.Close();            
		}// ParseHexFilePIC16()
       

        //---------------------------------------------------------------------
		// ParseHexFile8()
		//---------------------------------------------------------------------
		public void ParseHexFile8( clsParseSettings pobjSettings, int iTabLevel )
		{
            iProgWordsUsed = 0;
            iEEWordsUsed = 0;
            iConfigWordsUsed = 0;

            //--------------------------------------------------------------------------
			// Hex-file format
			// : 10 8000 00 0800 fa00 000f 8001
			// | |  |    |  |-Data
			// | |  |    |----Line record type
			// | |  |---------Line address
			// | |------------Line byte count
			// |--------------Start code (just a simple colon)
			//--------------------------------------------------------------------------
			int iLineAddrHigh = 0;
            int iAddress;
            int iOutAddr;
			
            string strFileLine;
			string strLineStartCode;
            int iLineByteCount;
            int iLineAddress;
            HexRecordType iLineRecordType;
            string strLineData;

            // Open hex-file
            TextReader textReader = new StreamReader( strFilename );            
 
            while ( textReader.Peek() != -1 ) {
                strFileLine = textReader.ReadLine();
                strLineStartCode = strFileLine.Substring(0, 1);

               if ( strLineStartCode == ":" ) {
                    //Parse the line
                    iLineByteCount = int.Parse( strFileLine.Substring( 1, 2 ), System.Globalization.NumberStyles.HexNumber );
                    iLineAddress = ( int.Parse( strFileLine.Substring( 3, 4 ), System.Globalization.NumberStyles.HexNumber ) & 65535 );
                    iLineRecordType = (HexRecordType)int.Parse( strFileLine.Substring( 7, 2 ), System.Globalization.NumberStyles.HexNumber );
                    strLineData = strFileLine.Substring( 9, ( iLineByteCount * 2 ) );

                    // Calculate address
                    iAddress = (iLineAddrHigh << 16) + iLineAddress;

                    // Data record
                    if ( iLineRecordType == HexRecordType.Data ) {

                        // Regular program memory
                        if ( iAddress <= pobjSettings.device.flashSizeP ) {
							iOutAddr = iAddress;
                            iProgWordsUsed += iLineByteCount/2;

						    for ( int iByte = 0; iByte < iLineByteCount; iByte++ ) {
							    iProgMem[iOutAddr + iByte] = int.Parse( strLineData.Substring(iByte*2 + 0, 2), System.Globalization.NumberStyles.HexNumber );
                                bProgPageUsed[ (iAddress + iByte) / (pobjSettings.device.pageSizeR * pobjSettings.device.rowsizeW * 2)  ] = true;
							    bProgRowUsed[  (iAddress + iByte) / (pobjSettings.device.rowsizeW * 2)  ] = true;
						    }	
                            

						// Eeprom memory
						} else if ( pobjSettings.device.EepromSizeB > 0 && iAddress >= 0xF00000 && iAddress < (0xF00000 + pobjSettings.device.EepromSizeB) ) {
                            iOutAddr = iAddress - 0xF00000;

							for ( int iByte = 0; iByte < iLineByteCount; iByte++ ) {
								iEEMem[ iOutAddr + iByte ] = int.Parse( strLineData.Substring(iByte*2 + 0, 2), System.Globalization.NumberStyles.HexNumber );								
								bEEWordUsed[ iOutAddr + iByte ] = true;
                                ++iEEWordsUsed;
							}

							                        
						// Config memory
						} else if ( pobjSettings.device.configCount > 0 && iAddress >= 0x300000 && iAddress <= 0x30000D) {	
							iOutAddr = iAddress - 0x300000;
							
							for ( int iByte = 0; iByte < iLineByteCount; iByte++ ) {
								iConfigMem[ iOutAddr + iByte ] = int.Parse( strLineData.Substring(iByte*2 + 0, 2), System.Globalization.NumberStyles.HexNumber );
								bConfigWordUsed[ iOutAddr + iByte ] = true;
                                ++iConfigWordsUsed;
							}	
                        
                        // Unknown data
                        } else {
                            OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Unknown data found, " + iLineByteCount.ToString() + " bytes at 0x" + iAddress.ToString("X4"), iTabLevel) );
                        }

                    // End Of File record
                    } else if ( iLineRecordType == HexRecordType.EndOfFile ) {
                        break;

                    // Extended Linear Address Record
                    // The two data bytes represent the upper 16 bits of the 32 bit address, when combined with the address of the 00 type record.
                    } else if ( iLineRecordType == HexRecordType.ExtendedLinearAddress ) {
                        iLineAddrHigh = ( int.Parse( strFileLine.Substring( 9, 4 ), System.Globalization.NumberStyles.HexNumber ) & 65535 );
                    }
                }
            }

            // Close files
            textReader.Close();            
		}// ParseHexFile8()
        

        //---------------------------------------------------------------------
		// ParseHexFile16()
		//---------------------------------------------------------------------
		public void ParseHexFile16( clsParseSettings pobjSettings, int iTabLevel )
		{
            iProgWordsUsed = 0;
            iEEWordsUsed = 0;
            iConfigWordsUsed = 0;

            //--------------------------------------------------------------------------
			// Hex-file format
			// : 10 8000 00 0800 fa00 000f 8001
			// | |  |    |  |-Data
			// | |  |    |----Line record type
			// | |  |---------Line address
			// | |------------Line byte count
			// |--------------Start code (just a simple colon)
			//--------------------------------------------------------------------------
			int iLineAddrHigh = 0;
            int iAddress;
            int iOutAddr;
			int iWordCount;	
			
            string strFileLine;
			string strLineStartCode;
            int iLineByteCount;
            int iLineAddress;
            HexRecordType iLineRecordType;
            string strLineData;

			// Open hex-file
			TextReader textReader = new StreamReader( strFilename );

			while ( textReader.Peek() != -1 ) {
				strFileLine = textReader.ReadLine();	        
                strLineStartCode = strFileLine.Substring(0, 1);
				
				if ( strLineStartCode == ":" ) {
					//Parse the line
					iLineByteCount = int.Parse( strFileLine.Substring(1, 2), System.Globalization.NumberStyles.HexNumber );
					iLineAddress = ( int.Parse(strFileLine.Substring(3, 4), System.Globalization.NumberStyles.HexNumber) & 65535 );
					iLineRecordType = (HexRecordType)int.Parse( strFileLine.Substring(7, 2), System.Globalization.NumberStyles.HexNumber );
					strLineData = strFileLine.Substring( 9, (iLineByteCount * 2) );
					
                    // Calculate address
					iAddress = ( (iLineAddrHigh << 16) + iLineAddress ) / 2;

					// Data record
					if ( iLineRecordType == HexRecordType.Data ) {
						iWordCount = iLineByteCount / 4;

                        if ( (iLineByteCount % 4) != 0 ) {
                            OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.warning, "Missaligned data, please contact the author at micke82@hotmail.com", iTabLevel) );
                        }

						// Regular program memory
						if ( iAddress <= pobjSettings.device.flashSizeP ) {
							iOutAddr = (iAddress * 3) / 2/*32bit-16bit*/;
						    for ( int iWord = 0; iWord < iWordCount; iWord++ ) {
							    //upper-low-high								
							    iProgMem[iOutAddr + iWord*3 + 0] = int.Parse( strLineData.Substring(iWord*8 + 4, 2), System.Globalization.NumberStyles.HexNumber );
							    iProgMem[iOutAddr + iWord*3 + 1] = int.Parse( strLineData.Substring(iWord*8 + 0, 2), System.Globalization.NumberStyles.HexNumber );
							    iProgMem[iOutAddr + iWord*3 + 2] = int.Parse( strLineData.Substring(iWord*8 + 2, 2), System.Globalization.NumberStyles.HexNumber );
							    ++iProgWordsUsed;
                                bProgPageUsed[ (iAddress/2 + iWord) / (pobjSettings.device.pageSizeR * pobjSettings.device.rowsizeW)  ] = true;
							    bProgRowUsed[  (iAddress/2 + iWord) / pobjSettings.device.rowsizeW  ] = true;
                            }
							 
						// Eeprom memory
						} else if ( pobjSettings.device.EepromSizeB > 0 && iAddress >= (0x800000 - pobjSettings.device.EepromSizeB) && iAddress < 0x800000 ) {
							iOutAddr = iAddress - ( 0x800000 - pobjSettings.device.EepromSizeB) ;
							
							for ( int iWord = 0; iWord < iWordCount; iWord++ ) {
								iEEMem[iOutAddr + iWord*2 + 0] = int.Parse( strLineData.Substring(iWord*8 + 0, 2), System.Globalization.NumberStyles.HexNumber );
								iEEMem[iOutAddr + iWord*2 + 1] = int.Parse( strLineData.Substring(iWord*8 + 2, 2), System.Globalization.NumberStyles.HexNumber );								
								bEEWordUsed[ iOutAddr/2 + iWord ] = true;
                                ++iEEWordsUsed;
							}
							                        
						// Config memory
						} else if ( pobjSettings.device.configCount > 0 && iAddress >= 0xF80000 && iAddress <= 0xF8000E) {	
							iOutAddr = iAddress - 0xF80000;
							
							for ( int iWord = 0; iWord < iWordCount; iWord++ ) {
								iConfigMem[iOutAddr + iWord*2 + 0] = int.Parse( strLineData.Substring(iWord*8 + 0, 2), System.Globalization.NumberStyles.HexNumber );
								iConfigMem[iOutAddr + iWord*2 + 1] = int.Parse( strLineData.Substring(iWord*8 + 2, 2), System.Globalization.NumberStyles.HexNumber );								
								bConfigWordUsed[iOutAddr/2 + iWord] = true;
                                ++iConfigWordsUsed;
							}	

                        // Unknown data
                        } else {
                            OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Unknown data found, " + iLineByteCount.ToString() + " bytes at 0x" + iAddress.ToString("X4"), iTabLevel) );
                        }
					
					// End Of File record
					} else if ( iLineRecordType == HexRecordType.EndOfFile ) {
							break;
					
					// Extended Linear Address Record
					// The two data bytes represent the upper 16 bits of the 32 bit address, when combined with the address of the 00 type record.
					} else if ( iLineRecordType == HexRecordType.ExtendedLinearAddress ) {
						iLineAddrHigh = ( int.Parse(strFileLine.Substring(9, 4), System.Globalization.NumberStyles.HexNumber) & 65535 );
					}
				}
			}

            textReader.Close();
		}// ParseHexFile16()


        //---------------------------------------------------------------------
		// ParseHexFile()
		//---------------------------------------------------------------------
		virtual public void ParseHexFile( clsParseSettings pobjSettings, int iTabLevel, ref bool pbResult )
		{
			pbResult = false;
            return;
		}// ParseHexFile()


        //---------------------------------------------------------------------
		// SetGotoToBl()
		//---------------------------------------------------------------------
		public void SetGotoToBl( clsParseSettings pobjSettings, int iTabLevel )
		{           
            string strFamily = pobjSettings.device.family.name;
            
            
            //---------------------------------------------------------------------
		    // PIC16F
		    //---------------------------------------------------------------------            
            if ( strFamily == "PIC16F" ) {
                if ( bProgPageUsed[0] == true || bProgRowUsed[0] == true ) {
                    if ( pobjSettings.noGoto == false ) {
                        int iBootloaderStartAddress = GetBootloaderStartAddress( pobjSettings );
                        int iPCLATH = (iBootloaderStartAddress >> 11); 
                        
                        // Word 0: page select, PCLATH 3
                        if ( (iPCLATH & 1) == 0 ) {
                            iProgMem[0] = 0x8A; iProgMem[1] = 0x11; //bcf  PCLATH, 3
                        } else {                            
                            iProgMem[0] = 0x8A; iProgMem[1] = 0x15; //bsf  PCLATH, 3
                        }
                        
                        // Word 1: page select, PCLATH 3
                        if ( (iPCLATH & 2) == 0 ) {                            
                            iProgMem[2] = 0x0A; iProgMem[3] = 0x12; //bcf  PCLATH, 4
                        } else {                            
                            iProgMem[2] = 0x0A; iProgMem[3] = 0x16; //bsf  PCLATH, 4
                        }
                        
                        // Word 2: goto to bootloader
                        iProgMem[4] = (iBootloaderStartAddress & 0x0000FF);             //low address byte
                        iProgMem[5] = 0x28 + ((0x0700 & iBootloaderStartAddress) >> 8);  //goto instruction
                    }
                } else {
                    OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.warning, "Row/page 0 is not found in hex-file, goto at 0x00 is not written.", iTabLevel) );
                }
            

            //---------------------------------------------------------------------
		    // PIC18F  PIC18FJ
		    //---------------------------------------------------------------------            
            } else if ( strFamily == "PIC18F" || strFamily == "PIC18FJ" ) {
                if ( bProgPageUsed[0] == true || bProgRowUsed[0] == true ) {
                    if ( pobjSettings.noGoto == false ) {
                        int iBootloaderStartAddress = GetBootloaderStartAddress( pobjSettings ) / 2;
                        // First goto word
                        iProgMem[0] = ((iBootloaderStartAddress & 0x0000FF));                //low address byte			
                        iProgMem[1] = 0xEF;                                                 //goto instruction
                        // Second goto word												
                        iProgMem[2] = ((iBootloaderStartAddress & 0x00FF00) >> 8);	        //high address byte
                        iProgMem[3] = 0xF0/*goto instruction*/ + ((iBootloaderStartAddress & 0x0F0000) >> 16);	//goto instruction & upper address nibble
                    }
                } else {
                    OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.warning, "Row/page 0 is not found in hex-file, goto at 0x00 is not written.", iTabLevel) );
                }            
                
            //---------------------------------------------------------------------
		    // PIC24F  PIC24FJ  PIC24H  dsPIC30F  dsPIC33FJ
		    //---------------------------------------------------------------------            
            } else {
                if ( bProgPageUsed[0] == true || bProgRowUsed[0] == true ) {
                    if ( pobjSettings.noGoto == false ) {
                        int iBootloaderStartAddress = GetBootloaderStartAddress( pobjSettings );
				        // First goto word
				        iProgMem[0] = 0x04;												//goto instruction
				        iProgMem[1] = ( (iBootloaderStartAddress & 0x0000FE) );			//low Address byte			
				        iProgMem[2] = ( (iBootloaderStartAddress & 0x00FF00) >> 8 );		//high Address byte				
				        // Second goto word
				        iProgMem[3] = 0x00;												//not used by goto
				        iProgMem[4] = ( (iBootloaderStartAddress & 0x7F0000) >> 16 );	//upper Address byte
				        iProgMem[5] = 0x00;												//not used by goto	
                    }
                } else {
                    OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.warning, "Row/page 0 is not found in hex-file, goto at 0x00 is not written.", iTabLevel) );
                }
            }
        }// SetGotoToBl()


        //---------------------------------------------------------------------
		// SizeBuffers()
		//---------------------------------------------------------------------
		public void SizeBuffers( 
            int piProgPageUsedBufferSize, int piProgRowUsedBufferSize, int piProgMemBufferSize, 
            int piEEWordsUsedBufferSize, int piEEMemBufferSize, 
            int piConfigWordsUsedBufferSize, int piConfigMemBufferSize 
        ) { 
			bProgPageUsed = new bool[ piProgPageUsedBufferSize ];
            bProgRowUsed = new bool[ piProgRowUsedBufferSize ];
            iProgMem = new int[ piProgMemBufferSize ];
			
			bEEWordUsed = new bool[ piEEWordsUsedBufferSize ];			
			iEEMem = new int[ piEEMemBufferSize ];
			
			bConfigWordUsed = new bool[ piConfigWordsUsedBufferSize ];
			iConfigMem = new int[ piConfigMemBufferSize ];	
        }// SizeBuffers()


        //---------------------------------------------------------------------
		// SwapGoto()
        // Description: swap user goto at 0x00 to bootloader usr:, se firmware code
        //              it´s important that this instruction is placed just before a new program row
		//---------------------------------------------------------------------
		public void SwapGoto( clsParseSettings pobjSettings, int piTabLevel )
		{           
            string strFamily = pobjSettings.device.family.name;

            //---------------------------------------------------------------------
		    // PIC16F
		    //---------------------------------------------------------------------            
            if ( strFamily == "PIC16F" ) {
                if ( pobjSettings.noGoto == false ) {
                    int iGotoUserAppAddress = GetBootloaderStartAddress( pobjSettings ) - 3;    //buffer index

                    //
                    iProgMem[iGotoUserAppAddress*2 + 0] = 0x8A; iProgMem[iGotoUserAppAddress*2 + 1] = 0x11; //bcf  PCLATH, 3
                    iProgMem[iGotoUserAppAddress*2 + 2] = 0x0A; iProgMem[iGotoUserAppAddress*2 + 3] = 0x12; //bcf  PCLATH, 4

                    // Find and move goto from word 0-2
                    int iOffset = 0;
                    if ( (iProgMem[1] & 0x38) == 0x28 ) {
                        iOffset = 0;
                    } else if  ( (iProgMem[3] & 0x38) == 0x28 ) {
                        iOffset = 2;
                    } else if  ( (iProgMem[5] & 0x38) == 0x28 ) {
                        iOffset = 4;
                    }
                    for ( int iIter = 0; iIter < 2; iIter++ ) {
                        iProgMem[iGotoUserAppAddress*2 + 4 + iIter] = iProgMem[iIter + iOffset];
                    } 

                    bProgPageUsed[iGotoUserAppAddress / (pobjSettings.device.pageSizeR * pobjSettings.device.rowsizeW)] = true;
                    bProgRowUsed[iGotoUserAppAddress / (pobjSettings.device.rowsizeW)] = true;

                    /*int iUserAppAddress = ((iProgMem[1]&0x3)<<8) + iProgMem[0];
                    int iPCLATH = (iUserAppAddress >> 11); 
                    
                    // Word 0: page select, PCLATH 3                    
                    if ( (iPCLATH & 1) == 0 ) {
                        iProgMem[iGotoUserAppAddress*2 + 0] = 0x8A; iProgMem[iGotoUserAppAddress*2 + 1] = 0x11; //bcf  PCLATH, 3
                    } else {                            
                        iProgMem[iGotoUserAppAddress*2 + 0] = 0x8A; iProgMem[iGotoUserAppAddress*2 + 1] = 0x15; //bsf  PCLATH, 3
                    }
                    
                    // Word 1: page select, PCLATH 4
                    if ( (iPCLATH & 2) == 0 ) {                            
                        iProgMem[iGotoUserAppAddress*2 + 2] = 0x0A; iProgMem[iGotoUserAppAddress*2 + 3] = 0x12; //bcf  PCLATH, 4
                    } else {                            
                        iProgMem[iGotoUserAppAddress*2 + 2] = 0x0A; iProgMem[iGotoUserAppAddress*2 + 3] = 0x16; //bsf  PCLATH, 4
                    }
                    //OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.warning, "Page 0 for user appl. entry point is assumed", piTabLevel) );                

                    // Word 2: move goto instruction from 0x00
                    for ( int iIter = 0; iIter < 2; iIter++ ) {
                        iProgMem[iGotoUserAppAddress*2 + iIter + 4] = iProgMem[iIter];
                    }
                    bProgPageUsed[iGotoUserAppAddress / (pobjSettings.device.pageSizeR * pobjSettings.device.rowsizeW)] = true;
                    bProgRowUsed[iGotoUserAppAddress / (pobjSettings.device.rowsizeW)] = true;*/                    
                    
                    // Move word 0-2
                    /*for ( int iIter = 0; iIter < 6; iIter++ ) {
                        iProgMem[iGotoUserAppAddress*2 + iIter] = iProgMem[iIter];
                    }
                    bProgPageUsed[iGotoUserAppAddress / (pobjSettings.device.pageSizeR * pobjSettings.device.rowsizeW)] = true;
                    bProgRowUsed[iGotoUserAppAddress / (pobjSettings.device.rowsizeW)] = true;
                    

                    if ( clsds30Loader.debugMode == true ) {                    
                        //int iUserAppAddress = ((iProgMem[1]&0x3)<<8) + iProgMem[0];
                        OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Goto user app address: 0x" + iGotoUserAppAddress.ToString("X2"), piTabLevel) );                        
                        //OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "User app address: 0x" + iUserAppAddress.ToString("X2"), piTabLevel) );
                    }*/
                }
            

            //---------------------------------------------------------------------
		    // PIC18F  PIC18FJ
		    //---------------------------------------------------------------------            
            } else if ( strFamily == "PIC18F" || strFamily == "PIC18FJ" ) {
                if ( pobjSettings.noGoto == false ) {
                    int iGotoUserAppAddress = GetBootloaderStartAddress( pobjSettings ) - 4;    //buffer index
                    for ( int iIter = 0; iIter < 4; iIter++ ) {
                        iProgMem[iGotoUserAppAddress + iIter] = iProgMem[iIter];
                    }
                    bProgPageUsed[iGotoUserAppAddress / (pobjSettings.device.pageSizeR * pobjSettings.device.rowsizeW * 2)] = true;
                    bProgRowUsed[iGotoUserAppAddress / (pobjSettings.device.rowsizeW * 2)] = true;

                    if ( clsds30Loader.debugMode == true ) {
                        int iUserAppAddress = ((iProgMem[iGotoUserAppAddress+3]&0xf)<<16) + iProgMem[iGotoUserAppAddress];
                        OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Goto user app address: 0x" + iGotoUserAppAddress.ToString("X2"), piTabLevel) );
                        OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "User app address: 0x" + iUserAppAddress.ToString("X2"), piTabLevel) );
                    }
                }
          
                
            //---------------------------------------------------------------------
		    // PIC24F  PIC24FJ  PIC24H  dsPIC30F  dsPIC33FJ
		    //---------------------------------------------------------------------            
            } else { 
                if ( pobjSettings.noGoto == false ) {
                    int iGotoUserAppAdress = GetBootloaderStartAddress( pobjSettings ) - 4;      
                    int iGotoUserAppAdressB3 = (iGotoUserAppAdress / 2) * 3;	//starting index in buffer

				    for ( int iIter = 0; iIter < 6; iIter++ ) {
					    iProgMem[ iGotoUserAppAdressB3 + iIter ] = iProgMem[ iIter ];
				    }
                    bProgPageUsed[iGotoUserAppAdress / (pobjSettings.device.pageSizeR * pobjSettings.device.rowsizeW * 2)] = true;
				    bProgRowUsed[ iGotoUserAppAdress / (pobjSettings.device.rowsizeW * 2) ] = true;

                    if ( clsds30Loader.debugMode == true ) {
                        int iUserAppAddress = (iProgMem[iGotoUserAppAdressB3+4]<<16) + (iProgMem[iGotoUserAppAdressB3+2]<<8) + iProgMem[iGotoUserAppAdressB3+1];
                        OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "Goto user app address: 0x" + iGotoUserAppAdress.ToString("X2"), piTabLevel) );
                        OnHexFileParse( new clsHexFileParseEventArgs(clsHexFileParseEventArgs.EventType.info, "User app address: 0x" + iUserAppAddress.ToString("X2"), piTabLevel) );
                    }
			    }
            }
        }// SwapGoto()


  		//---------------------------------------------------------------------
		// UpdateParsedSettings()
		//---------------------------------------------------------------------
        protected void UpdateParsedSettings( clsParseSettings pobjSettings )         
        {
            bIsParsed = true;
            objParsedSettings.CopyFrom( pobjSettings );
        }// UpdateParsedSettings()


        //---------------------------------------------------------------------
		// ValidateHexFile()
		// Description: validates an Intel hex-file
        //--------------------------------------------------------------------------
		// Hex-file format
		// : 10 8000 00 0800 fa00 000f 8001
		// | |  |    |  |-Data
		// | |  |    |----Line record type
		// | |  |---------Line address
		// | |------------Line byte count
		// |--------------Start code (just a simple colon)
		//---------------------------------------------------------------------
		public void ValidateHexFile( int iTabLevel, ref bool pbResult )
		{	
			OnHexFileValidate( new clsHexFileValidateEventArgs(clsHexFileValidateEventArgs.EventType.started, "Validating hexfile...", iTabLevel++ ) );
		    
			//--------------------------------------------------------------------------
			// Check file existense
			//--------------------------------------------------------------------------
			if ( strFilename == string.Empty || File.Exists(strFilename) == false) {
				OnHexFileValidate( new clsHexFileValidateEventArgs(clsHexFileValidateEventArgs.EventType.failed, "nonexisting file specified", -1) );
				pbResult = false;
                return;
			}
		    
		    
			//--------------------------------------------------------------------------
			// Create file reader object
			//--------------------------------------------------------------------------
		    TextReader textReader = new StreamReader( strFilename );
		    
		    
			//--------------------------------------------------------------------------
			// Check for an empty file
			//--------------------------------------------------------------------------
			if ( textReader.Peek() == -1 ) {
                OnHexFileValidate( new clsHexFileValidateEventArgs(clsHexFileValidateEventArgs.EventType.failed, "hex-file is empty", -1) );
				textReader.Close();
				pbResult = false;
                return;
			}
		    
		    
			//--------------------------------------------------------------------------
			// Do validation
			//--------------------------------------------------------------------------
			int iIter;            
            int iChecksum;
			string strInFileLine;
            int iLineNr;

            string strStartCode;
            int iLineByteCount;
			int iLineAddress;
            HexRecordType iLineRecordType;
			string strLineData;
            int iLineChecksum;			

            iLineNr = 0;
            while ( textReader.Peek() != -1 ) {
				strInFileLine = textReader.ReadLine();
                ++iLineNr;
                strStartCode = strInFileLine.Substring(0, 1);


                //---------------------------------------------------------
                //
                //---------------------------------------------------------
				if ( strStartCode == ":" ) {	

				    //---------------------------------------------------------
                    // Parse the line
                    //---------------------------------------------------------
					iLineByteCount = int.Parse( strInFileLine.Substring(1, 2), System.Globalization.NumberStyles.HexNumber );
					iLineAddress = int.Parse( strInFileLine.Substring(3, 4), System.Globalization.NumberStyles.HexNumber ) & 65535;
                    iLineRecordType = (HexRecordType)int.Parse( strInFileLine.Substring(7, 2), System.Globalization.NumberStyles.HexNumber );
	                strLineData = strInFileLine.Substring( 9, (iLineByteCount * 2) );
                    iLineChecksum = int.Parse( strInFileLine.Substring(9+(iLineByteCount*2), 2), System.Globalization.NumberStyles.HexNumber );
	            

                    //---------------------------------------------------------
					// Check crc
                    //---------------------------------------------------------
                    iChecksum = 0;
                    int i8bitWords = (strInFileLine.Length-1-2) / 2;
					for ( iIter = 0; iIter < i8bitWords; iIter++ ) {
						iChecksum += int.Parse( strInFileLine.Substring((2 * iIter) + 1, 2), System.Globalization.NumberStyles.HexNumber ) & 0xFF;
					}           
					if ( ((iChecksum + iLineChecksum) & 0xff) != 0 ) {
                        OnHexFileValidate( new clsHexFileValidateEventArgs(clsHexFileValidateEventArgs.EventType.failed, "crc-error on line " + iLineNr.ToString(), -1) );
						textReader.Close();
						pbResult = false;
                        return;
					}

                    
                    //---------------------------------------------------------
					// End Of File record
                    //---------------------------------------------------------
					if ( iLineRecordType == HexRecordType.EndOfFile ) {
						break;

                    //---------------------------------------------------------
					// Unsupported hex record type?
                    //---------------------------------------------------------
                    } else if ( iLineRecordType != HexRecordType.ExtendedLinearAddress && iLineRecordType != HexRecordType.Data ) {
                        OnHexFileValidate( new clsHexFileValidateEventArgs(clsHexFileValidateEventArgs.EventType.failed, "Unsupported hex record type found: " + iLineRecordType.ToString(), iTabLevel+1) );
                    }


                //---------------------------------------------------------
                //This shouldn't happen(?)
                //---------------------------------------------------------
				} else if ( strStartCode == " " || strStartCode == "\t" || strStartCode == "\r" || strStartCode == "\n" ) {
                
                    
                //---------------------------------------------------------
                // Unknown starting character
                //---------------------------------------------------------
				} else {
                    OnHexFileValidate( new clsHexFileValidateEventArgs(clsHexFileValidateEventArgs.EventType.failed, "invalid start code: \"" + strStartCode + "\" on line " + iLineNr.ToString(), -1) );
					textReader.Close();
					pbResult = false;
                    return;
				}

		        
                //---------------------------------------------------------
                // End of file?
                //---------------------------------------------------------
				if ( textReader.Peek() == -1 ) {
                    OnHexFileValidate( new clsHexFileValidateEventArgs(clsHexFileValidateEventArgs.EventType.failed, "missing end of file record", -1) );
					textReader.Close();
					pbResult = false;
                    return;
				}
			} //while ( textReader.Peek() != -1 ) {
		    

            //            
            textReader.Close();

            //
            OnHexFileValidate( new clsHexFileValidateEventArgs(clsHexFileValidateEventArgs.EventType.success, "", -1) );		    
			pbResult = true;
            return;
		}// ValidateHexFile()
 
	}// Class: clsHex    
}
